/* scriptris - a free j2me tetris (R) clone with
 * bluetooth multiplayer support
 *
 *
 * (c) 2005-2006 Michael "ScriptKiller" Arndt
 * <scriptkiller@gmx.de>
 * http://scriptkiller.de/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 */


import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

import java.util.Vector;

public class Tetris extends MIDlet {

    public static final int SINGLE=0;
    public static final int MULTI_HOST=1;
    public static final int MULTI_CLIENT=2;
    public static final int MULTI_TCP_CLIENT=3;

    public static final int TYPE_NONE=0;
    public static final int TYPE_BT=1;
    public static final int TYPE_TCP=2;
    


    Display display;
    mainCanvas gameCanvas;
    configCanvas confCanvas;
    gamePartyLine p;
    About about;
    Help help;

    private brickTimer timer;
    private Bluetooth bt;
    private TCP tcp;

    public int clientType=TYPE_NONE; /* TYPE_BT or TYPE_TCP */
    public int gameType=SINGLE;

    /* only used in multi-player: has a game ended? */
    public boolean gameEnded=true;


    //    public int serverPort=6623;
    public int serverPort=6000;
    //    public String serverHostName="scriptkiller.de";
    public String serverHostName="scriptkiller.shacknet.nu";

    public String nickName="Test";


    public Tetris() {
	display=Display.getDisplay(this);
	p=new gamePartyLine(this);
    }


    public void startApp() {
	showModeSelect();
    }

    public void showAbout() {
	about=new About(this);
	display.setCurrent(about);
    }

    public void showHelp() {
	help=new Help(this);
	display.setCurrent(help);
    }

    public void showModeSelect() {
	gameModeSelect gs=new gameModeSelect(this);
	display.setCurrent(gs);
    }

    public void showServerConfig() {
	gameServerConfig c=new gameServerConfig(this, bt.getLocalDevName());
	display.setCurrent(c);
    }

    public void showClientConfig() {
	gameClientConfig c=new gameClientConfig(this);
	display.setCurrent(c);
    }

    public void showPartyLine() {
	p.init();
	display.setCurrent(p);
    }

    public void startServer(String name) {
	
	bt.setServerName(name);
	bt.startServer();
	bt.listen();
	showPartyLine();
	p.showWaitingForPlayers();
    }

    public void startClient() {
	if(clientType == TYPE_BT) {

	    showPartyLine();
	    p.showSearching();

	    bt.startInq();
	}
	else if(clientType == TYPE_TCP) {
	    tcp.connect(serverHostName, serverPort);
	}
    }


    public void pauseApp() {
    }

    public void destroyApp(boolean unconditional) {
    }

    public void showInGameMenu() {

	/* only pause in single-player */
	if(gameType == SINGLE) {
	    pause();
	}

	gameMenu m=new gameMenu(this);
	display.setCurrent(m);
    }

    public void hideInGameMenu() {
	display.setCurrent(gameCanvas);

	/* only in single-player .. */
	if(gameType == SINGLE) {
	    pause(); /* unpause */
	}
    }

    public void pause() {

	if(timer != null && timer.running) {
	    timer.running=false;
	    timer=null;
	}
	else {
	    timer=new brickTimer(gameCanvas);
	    timer.start();
	}
    }

    public void quit() {
	try {
	    destroyApp(false);
	    notifyDestroyed();
	} catch(Exception e) {


	}
    }

    public void startGame(int gametype) {

	this.gameType=gametype;

	/* (re-)initialise game canvas and timer */
	gameCanvas=new mainCanvas(this);
	timer=new brickTimer(gameCanvas);

	/* game has not ended */
	gameEnded=false;


	/* if we have bluetooth multiplayer
	 * mode check and enable bluetooth now!
	 */

	if(gametype == SINGLE) {
	    display.setCurrent(gameCanvas);
	    timer.start();
	}
	else if(gametype == MULTI_HOST) {
	    if(initBluetooth()) {
		showServerConfig();
	    }
	}
	else if(gametype == MULTI_CLIENT) {
	    clientType=TYPE_BT;
	    if(initBluetooth()) {
		showClientConfig();
	    }
	}
	else if(gametype == MULTI_TCP_CLIENT) {
	    clientType=TYPE_TCP;
	    initTCP();
	    showClientConfig();
	}

    }

    /* only used in multiplayer:
     * request to restart game (same server, same client, ...)
     */
    public void restartGame() {
	send((byte)0x0C); /* restart request */
	restartGameStart();
    }


    /* locally restart the game */
    public void restartGameStart() {

	if(timer!=null) {
	    timer.running=false;
	}
	timer=null;

	/* (re-)initialise game canvas and timer */
	gameCanvas=new mainCanvas(this);
	timer=new brickTimer(gameCanvas);

	/* game has not ended */
	gameEnded=false;

	display.setCurrent(gameCanvas);
	timer.start();

    }

    public void initTCP() {

	tcp=new TCP(this);
	if(!tcp.init()) {
	    /* todo: kaputt? */
	    Alert a=new Alert("Fehler", "Kein TCP/IP!", null, null);
	    a.setTimeout(Alert.FOREVER);
	    display.setCurrent(a);
	}
    }

    public boolean initBluetooth() {

	bt=new Bluetooth(this);
	if(!bt.init()) {

	    Alert a=new Alert("Fehler", "Kein Bluetooth!", null, null);
	    a.setTimeout(Alert.FOREVER);
	    display.setCurrent(a);

	    return false;

	}

	return true;

    }

    /* bluetooth search is done */
    public void searchComplete(Server servers[]) {

	if(servers == null || servers.length==0) {
	    p.showNoServersFound();
	}
	else {
	    gameServerList g=new gameServerList(this, servers);
	    display.setCurrent(g);
	}

    }

    public void connectToServer(Server s) {
	bt.startClient(s);
    }

    public void connectedToServer() {

	if(clientType == TYPE_TCP) {
	    /* send nickname */
	    tcp.sendNickName(nickName);

	    p.showWaitingForPlayers();
	}
	else if(clientType == TYPE_BT) {
	    display.setCurrent(gameCanvas);
	    timer.start();
	}

    }

    /* a client has connected to our server */
    public void clientConnected() {
 	display.setCurrent(gameCanvas);
	timer.start();
    }

    public void recvEvent(byte b) {

	int number=0;

	switch(b) {

	case 0x03:
	    number=4;
	    gameCanvas.addRandomRows(number);
	    break;
	case 0x02:
	    number=2;
	    gameCanvas.addRandomRows(number);
	    break;
	case 0x01:
	    number=1;
	    gameCanvas.addRandomRows(number);
	    break;
	case 0x07:
	    /* game has ended */
	    gameEnded=true;

	    gameCanvas.showLost();
	    break;
	case 0x08:
	    /* game has ended */
	    gameEnded=true;

	    gameCanvas.showWon();
	    break;
	case 0x05:
	    /* peer lost the game */
	    send((byte)0x07); /* you loose */
	    gameCanvas.showWon(); /* we win :) */

	    /* game has ended */
	    gameEnded=true;

	    break;
	case 0x09:
	    /* disconnection from peer */
	    pause(); // TODO: is this pause ok?
	    showPartyLine();
	    p.showConnectionError("Gegenstelle hat Verbindung beendet");
	    break;
	case 0x0A:
	    /* ping received, send pong */
	    send((byte)0x0B);
	    break;
	case 0x0B:
	    /* pong received, feeling happy :) */
	    break;
	case 0x0C:
	    /* restart game */
	    restartGameStart();
	    break;
	}


    }

    /* hit the ceiling */
    public void endOfGame() {
	pause();

	/* game has ended */
	gameEnded=true;

	if(gameType == SINGLE) {
	    gameCanvas.showLost();
	}
	else if(gameType == MULTI_HOST) {
	    send((byte)0x08); /* you win */
	    gameCanvas.showLost(); /* we loose :( */
	}
	else if(gameType == MULTI_CLIENT || gameType == MULTI_TCP_CLIENT) {

	    if(clientType == TYPE_TCP) {
		/* wait for server to display message */
		send((byte)0x05);
	    }
	    else if(clientType == TYPE_BT) {
		send((byte)0x08); /* you win */
		gameCanvas.showLost(); /* we loose :( */
	    }
	}

    }


    /* multiple rows completed in one strike */
    public void multiRowCompleted(int count) {

	if(count >= 4) {
	    send((byte)0x03); /* add four */
	}
	else if(count == 3) {
	    send((byte)0x02); /* add two */
	}
	else if(count == 2) {
	    send((byte)0x01); /* add one */
	}

    }


    /* send byte to peer */
    public void send(byte b) {

	/* send over bt client */
	if(gameType == MULTI_CLIENT) {
	    //	    if(c!=null) {
	    bt.c.send(b);
	}
	
	/* send over bt server */
	if(gameType == MULTI_HOST) {
	    //	    if(bt.s!=null) {
	    bt.s.send(b);
	}

	/* send over tcp */
	if(clientType == TYPE_TCP) {
	    tcp.c.send(b);
	}

    }

    public void setServerHostName(String s) {
	this.serverHostName=s;
    }
    
    public void setServerPort(int i) {
	this.serverPort=i;
    }

    public void setNickName(String s) {
	this.nickName=s;
    }


    public void showConnecting() {
	showPartyLine();
	p.showConnecting();
    }

    public void connectionError(String s) {
	showPartyLine();
	p.showConnectionError(s);
    }

    public void nickNameReceived(String s) {
	/* we play against player s */
	/* start game */
		
	display.setCurrent(gameCanvas);
	timer.start();

    }

    /* stop running game,
     * stop searching,
     * stop server,
     * disconnect from server,
     * show mode select
     */
    public void stopGame() {

	if(gameType == SINGLE) {

	}
	else if(gameType == MULTI_CLIENT) {
	    bt.stopClient();
	}
	else if(gameType == MULTI_HOST) {
	    bt.stopServer();
	}
	else if(gameType == MULTI_TCP_CLIENT) {
	    tcp.disconnect();
	}

	showModeSelect();

    }



    /* send a keep alive packet to peer,
     * usually called by a timer, e.g. blockTimer
     */
    public void bluetoothKeepAlive() {

	if(gameType == MULTI_CLIENT ||
	   gameType == MULTI_HOST) {
	    send((byte)(0x0A));
	}

    }

}
