/* scriptris - a free j2me tetris (R) clone with
 * bluetooth multiplayer support
 *
 *
 * (c) 2005-2006 Michael "ScriptKiller" Arndt
 * <scriptkiller@gmx.de>
 * http://scriptkiller.de/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 */

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import java.util.Random;

public class mainCanvas extends Canvas implements CommandListener {

    public static final int ROTATE_LEFT=1;
    public static final int ROTATE_RIGHT=2;
    public static final int LEFT=3;
    public static final int RIGHT=4;
    public static final int STEP=5;
    public static final int DROP=6;

    public static final int COLS=12;
    public static final int ROWS=18;

    private int areaWidth;
    private int areaHeight;

    private int blockWidth, blockHeight;

    private Brick brick;

    private Row rows[];

    private Random rand;

    private Tetris t;

    private Scoring score;

    private boolean showLost=false;
    private boolean showWon=false;

    public mainCanvas(Tetris t) {
	this.t=t;

	rand=new Random();
	score=new Scoring();

	newBrick();

	rows=new Row[ROWS];
	for(int i=0;i<rows.length;i++) {
	    rows[i]=new Row(i);
	}

	/* full screen only in midp 2.0 */
	setFullScreenMode(true);

	Command exit = new Command("Beenden", Command.EXIT, 2);
	//        this.setCommandListener(this);
	//        this.addCommand(exit);


    }


    /*
    public void keyRepeated(int keyCode) {
	keyPressed(keyCode);
    }
    */

    public void keyPressed(int keyCode) {

	boolean useGameAction=false;

	System.err.println(keyCode);

	if(useGameAction) {

	    switch(getGameAction(keyCode)) {
	    case Canvas.UP:
		brickTransition(ROTATE_LEFT);
		break;
	    case Canvas.DOWN:
		brickTransition(DROP);
		break;
	    case Canvas.LEFT:
		brickTransition(LEFT);
		break;
	    case Canvas.RIGHT:
		brickTransition(RIGHT);
		break;
	    }
	}
	else {

	    /* no game action */
	    switch(keyCode) {
		
	    case Canvas.KEY_NUM1:
		brickTransition(ROTATE_LEFT);
		break;
	    case Canvas.KEY_NUM3:
		brickTransition(ROTATE_RIGHT);
		break;
	    case Canvas.KEY_NUM4:
		brickTransition(LEFT);
		break;
	    case Canvas.KEY_NUM6:
		brickTransition(RIGHT);
		break;
	    case Canvas.KEY_NUM5:
		brickTransition(STEP);
		break;
	    case Canvas.KEY_NUM7:
		brickTransition(DROP);
		break;

	    case -6: /* left softbutton */
		break;
	    case -7: /* right softbutton */
		break;
	    case -5: /* center button */
		t.showInGameMenu();
		break;
	    }
	}
    }


    public synchronized void brickTimerStep() {
	brickTransition(STEP);
	repaint();
    }

    public synchronized void brickTransition(int type) {

	/* make a copy of brick */
	Brick temp=brick.clone();

	if(type==ROTATE_LEFT) {
	    /* rotate copy */
	    temp.rotate(true);
	    /* now check for collisions */
	    if(!brickCollisionCheck(temp)) {
		/* no collision? -> rotate real brick */
		brick.rotate(true);
	    }
	}
	else if(type==ROTATE_RIGHT) {
	    /* rotate copy */
	    temp.rotate(false);
	    /* now check for collisions */
	    if(!brickCollisionCheck(temp)) {
		/* no collision? -> rotate real brick */
		brick.rotate(false);
	    }
	}
	else if(type==LEFT) {
	    temp.left();
	    if(!brickCollisionCheck(temp)) {
		brick.left();
	    }
	}
	else if(type==RIGHT) {
	    temp.right();
	    if(!brickCollisionCheck(temp)) {
		brick.right();
	    }
	}
	else if(type==STEP) {
	    temp.step();
	    if(!brickCollisionCheck(temp)) {
		brick.step();
	    }
	    else {
		brick.step();
		/* brick is now solid,
		 * get a new one */
		addBrickToRows(brick);
		rowCompleteCheck();
		newBrick();
		/* game lost? */
		if(brickCollisionCheck(brick)) {
		    t.endOfGame();	
		}
	    }
	}
	else if(type==DROP) {
	    while(!brickCollisionCheck(brick)) {
		brick.step();
	    }
	    addBrickToRows(brick);
	    rowCompleteCheck();
	    newBrick();
	    /* game lost? */
	    if(brickCollisionCheck(brick)) {
		t.endOfGame();	
	    }
	}
	
	temp=null;

	/* repaint! */
	repaint();

    }


    /* create new random brick */
    public void newBrick() {
	/* TODO: random is ugly */
	brick=new Brick(Math.abs((int)rand.nextLong()%7));
	brick.setPosition( ((int)COLS/2)-2, 0);
    }

    /* add the brick to the rows-Objects
     * so that brick-Object may be destroyed */
    public void addBrickToRows(Brick b) {
	int x, y;
	for(int i=0;i<b.blocks.length;i++) {
	    x=b.blocks[i].x;
	    y=b.blocks[i].y;

	    if(y <= 0) {
	
		return;
	    }

	    rows[y-1].blocks[x]=b.blocks[i];
	}
    }

    public boolean brickCollisionCheck(Brick b) {

	int i, y, x;
	for(i=0;i<b.blocks.length;i++) {

	    y=b.blocks[i].y;
	    x=b.blocks[i].x;

	    /* collision with ground */
	    if(y >= ROWS) {
		return true;
	    }

	    /* collision with left or right border */
	    if(x < 0 ||
	       x >= COLS) {
		return true;
	    }

	    /* collision with rows */
	    if(rows[y].blocks[x] != null) {
		return true;
	    }

	}

	return false;

    }


    public void rowCompleteCheck() {

	/* count number of completed rows in one step */
	int count=0;

	for(int y=0;y<rows.length;y++) {

	    boolean hasnull=false;

	    for(int x=0;x<rows[y].blocks.length;x++) {
		if(rows[y].blocks[x] == null) {
		    //		    System.err.println("Row "+y+" has null at "+x);

		    hasnull=true;
		    break;
		}
	    }

	    if(!hasnull) {
		/* row completed */
		System.err.println("Row "+y+" complete!");

		/* remove this row */

		/* move other rows "down" (_in_crease y!) */
		for(int i=y;i>0;i--) {
		    rows[i]=rows[i-1];
		}

		/* add new row on top */
		rows[0]=new Row(0);

		/* increase counter */
		count++;

	    }

	}


	/* add score */
	score.addLines(count);
	switch(count) {
	case 4: score.addScore(1200); break;
	case 3: score.addScore(300); break;
	case 2: score.addScore(100); break;
	case 1: score.addScore(40); break;
	}

	
	/* multi-player */
	if(count > 1) {
	    t.multiRowCompleted(count);
	}
	
    }


    public void paint_old(Graphics g) {

	int areaXoffset=0;
	int areaYoffset=0;

	/* background */
	g.setColor(0x000000);
	g.fillRect(0, 0, getWidth(), getHeight());

	/* game-area */
	areaHeight=getHeight();
	
	if(areaHeight%ROWS != 0) {
	    areaHeight-=areaHeight%ROWS;
	}

	/* calculate blockWidth + blockHeight*/
	blockWidth=blockHeight=areaHeight/ROWS;

	/* calculate areaWidth */
	areaWidth=COLS*blockWidth;

	g.setColor(0xFFffFF);
	g.fillRect(areaXoffset,areaYoffset, areaWidth, areaHeight);


	if(!showLost && !showWon) {
	    
	    
	    /* paint rows */
	    for(int y=0;y<rows.length;y++) {
		for(int x=0;x<rows[y].blocks.length;x++) {
		    
		    if(rows[y].blocks[x] == null) {
			continue;
		    }
		    
		    int color=rows[y].blocks[x].color;
		    int xpos=rows[y].blocks[x].x;
		    int ypos=rows[y].blocks[x].y;
		    
		    g.setColor(color);
		    g.fillRect(areaXoffset+x*blockWidth,
			       areaYoffset+y*blockHeight,
			       blockWidth,
			       blockHeight);
		}
	    }
	    
	    
	    /* paint brick */
	    for(int i=0;i<brick.blocks.length;i++) {
		int color=brick.blocks[i].color;
		int x=brick.blocks[i].x;
		int y=brick.blocks[i].y;
		
		g.setColor(color);
		g.fillRect(areaXoffset+x*blockWidth,
			   areaYoffset+y*blockHeight,
			   blockWidth,
			   blockHeight);
	    }
	}
	else if(showLost) {
	    /* game is lost */
	    g.setColor(0x000000);
	    g.drawString("Verloren!", areaXoffset+blockWidth*(COLS/2),
			 areaYoffset+blockHeight*(ROWS/2),
			 Graphics.TOP | Graphics.LEFT);

	}
	else if(showWon) {
	    /* game is won */
	    g.setColor(0x000000);
	    g.drawString("Gewonnen!", areaXoffset+blockWidth*(COLS/2),
			 areaYoffset+blockHeight*(ROWS/2),
			 Graphics.TOP | Graphics.LEFT);
	}


	/* draw small lines */
	g.setColor(0x999999);

	/* cols */
	for(int i=0;i<COLS;i++) {
	    g.drawLine(areaXoffset+i*blockWidth, areaYoffset,
		       areaXoffset+i*blockWidth, areaYoffset+areaHeight);
	}

	/* rows */
	for(int i=0;i<ROWS;i++) {
	    g.drawLine(areaXoffset, areaYoffset+i*blockHeight,
		       areaXoffset+areaWidth, areaYoffset+i*blockWidth);
	}


    }

    public void paint(Graphics g) {

	int areaXoffset=0;
	int areaYoffset=0;

	int scoreWidth, scoreHeight;
	int scoreXoffset;
	int scoreYoffset=0;

	/* background */
	g.setColor(0x000000);
	g.fillRect(0, 0, getWidth(), getHeight());

	/* game-area */
	//#ifdef CLDC_1.1
	//areaWidth=(int)( getWidth() * (2./3.) );
	//#else
	areaWidth=(getWidth()/3)*2;
	//#endif
	areaHeight=getHeight();


	System.err.println("Height: "+getHeight());

	//	if(areaWidth % COLS != 0) {
	//  areaWidth-=areaWidth%COLS;
	//}




	/* try to get the maximum out of available space
	 *
	 * first try maximum width */
	int test_width, test_height;
	int block_width, block_height;

	test_width=areaWidth;
	/* => block width */
	block_width=test_width/COLS;
	/* => block heigth */
	block_height=block_width;

	/* => area height */
	test_height=block_height*ROWS;

	System.out.println("h: "+test_height+" w: "+test_width+" b: "+block_height);

	if(test_height > areaHeight) {
	    /* bigger than maximum height?
	     * => try maximum height */

	    test_height=areaHeight;
	    /* => block height */
	    block_height=test_height/ROWS;
	    /* => block width */
	    block_width=block_height;

	    /* => area width */
	    test_width=block_width*COLS;

	    /* should be OK now! */
	    System.out.println("h: "+test_height+" w: "+test_width+" b: "+block_height);

	}

	System.out.println("h: "+test_height+" w: "+test_width+" b: "+block_height);


	areaHeight=block_height*ROWS;
	areaWidth=block_width*COLS;;
	blockHeight=block_height;
	blockWidth=block_width;


	System.err.println(areaWidth+"x"+areaHeight);
	System.err.println(blockWidth+"x"+blockHeight);


	/* score-area */
	scoreXoffset=areaWidth;
	//#ifdef CLDC_1.1
	//scoreWidth=(int)( getWidth() * (1./3.) );
	//#else
	scoreWidth=getWidth()/3;
	//#endif

	/* score height = area height */
	scoreHeight=areaHeight;

	
	/* calculate y-offsets, show everything in center of screen */
	areaYoffset=(getHeight()-areaHeight)/2;
	scoreYoffset=areaYoffset;


	/* draw game area */
	g.setColor(0xFFffFF);
	g.fillRect(areaXoffset,areaYoffset, areaWidth, areaHeight);

	/* draw score area */
	g.setColor(0xFFffFF);
	g.fillRect(scoreXoffset,scoreYoffset, scoreWidth, scoreHeight);


	if(!showLost && !showWon) {
	    
	    
	    /* paint rows */
	    for(int y=0;y<rows.length;y++) {
		for(int x=0;x<rows[y].blocks.length;x++) {
		    
		    if(rows[y].blocks[x] == null) {
			continue;
		    }
		    
		    int color=rows[y].blocks[x].color;
		    int xpos=rows[y].blocks[x].x;
		    int ypos=rows[y].blocks[x].y;
		    
		    g.setColor(color);
		    g.fillRect(areaXoffset+x*blockWidth,
			       areaYoffset+y*blockHeight,
			       blockWidth,
			       blockHeight);
		}
	    }
	    
	    
	    /* paint brick */
	    for(int i=0;i<brick.blocks.length;i++) {
		int color=brick.blocks[i].color;
		int x=brick.blocks[i].x;
		int y=brick.blocks[i].y;
		
		g.setColor(color);
		g.fillRect(areaXoffset+x*blockWidth,
			   areaYoffset+y*blockHeight,
			   blockWidth,
			   blockHeight);
	    }
	}


	/* draw small lines */
	g.setColor(0x999999);

	/* cols */
	for(int i=0;i<=COLS;i++) {
	    g.drawLine(areaXoffset+i*blockWidth, areaYoffset,
		       areaXoffset+i*blockWidth, areaYoffset+areaHeight);
	}

	/* rows */
	for(int i=0;i<ROWS;i++) {
	    g.drawLine(areaXoffset, areaYoffset+i*blockHeight,
		       areaXoffset+areaWidth, areaYoffset+i*blockHeight);
	}


	int fontAnchorX=areaXoffset+blockWidth*(COLS/2);
	int fontAnchorY=areaYoffset+blockHeight*(ROWS/2);


	if(showLost) {
	    drawCenteredTextBox(g, fontAnchorX, fontAnchorY,
				"Verloren!");
	}
	else if(showWon) {
	    drawCenteredTextBox(g, fontAnchorX, fontAnchorY,
				"Gewonnen!");
	}



	/* draw score */
	int fontHeight;
	fontHeight=g.getFont().getBaselinePosition();
	if(fontHeight <= 2) {
	    /* returns wrong values on 6230i */
	    fontHeight=g.getFont().getHeight();

	}
	g.setColor(0x000000);
	g.drawString(" Punkte: ",
		     scoreXoffset, scoreYoffset,
		     Graphics.TOP | Graphics.LEFT);

	g.drawString(" "+String.valueOf(score.getScore()),
		     scoreXoffset,
		     scoreYoffset+fontHeight,
		     Graphics.TOP | Graphics.LEFT);

	g.drawString(" Lines: ",
		     scoreXoffset, scoreYoffset+40,
		     Graphics.TOP | Graphics.LEFT);

	g.drawString(" "+String.valueOf(score.getLines()),
		     scoreXoffset,
		     scoreYoffset+40+fontHeight,
		     Graphics.TOP | Graphics.LEFT);


    }

    public void commandAction(Command c, Displayable d) {

    }


    /* show lost-game screen */
    public void showLost() {
	showLost=true;
	repaint();
    }

    /* show won-game screen */
    public void showWon() {
	showWon=true;
	repaint();
    }

    /* add 'count' lines at bottom of game */
    public void addRandomRows(int count) {

	Row r;

	for(int i=0;i<count;i++) {

	    /* new row at ypos ROWS-1 (bottom) */
	    r=new Row(ROWS-1);
	    for(int z=0;z<r.blocks.length;z++) {
		/* random: block here? */
		/* TODO: random is ugly */
		if(Math.abs((int)rand.nextLong()%6) != 0) {
		    int color=Brick.colors[Math.abs((int)rand.nextLong()%7)];
		    /* random color, xpos = z, ypos = ROWS-1 */
		    r.blocks[z]=new Block(color, z, ROWS-1); 
		}
	    }

	    /* move other rows up */
	    for(int y=0;y<ROWS-1;y++) {
		rows[y]=rows[y+1];
	    }

	    /* replace it */
	    rows[ROWS-1]=r;

	}

	/* TODO: do a end of game check? */

	/* repaint! */
	repaint();

    }

    public void drawCenteredTextBox(Graphics g,
				    int fontAnchorX,
				    int fontAnchorY,
				    String s) {

	Font f=g.getFont();

	/* TODO: see above ... */
	int fontHeight;
	fontHeight=f.getBaselinePosition();
	if(fontHeight <= 2) {
	    /* returns wrong values on 6230i */
	    fontHeight=f.getHeight();

	}


	/* background */
	g.setColor(0xFFFFFF);
	g.fillRect(fontAnchorX-
		   f.stringWidth(s)/2 - 5,
		   
		   fontAnchorY - 5,
		   f.stringWidth(s)+10,
		   fontHeight+10);
	
	/* draw box outline */
	g.setColor(0x999999);
	g.drawRect(fontAnchorX-
		   f.stringWidth(s)/2 - 5,
		   
		   fontAnchorY - 5,
		   f.stringWidth(s)+10,
		   fontHeight+10);
	
	
	/* draw string */
	g.setColor(0x000000);
	g.drawString(s,
		     fontAnchorX,
		     fontAnchorY,
		     Graphics.TOP | Graphics.HCENTER);
    }

    /* see class Tetris */
    public void bluetoothKeepAlive() {
	t.bluetoothKeepAlive();
    }

}


